record_full = '{"action": "record", "format": "wav", "split": "conversation", "channels": 4, "endOnSilence": 5, "endOnKey": "*", "timeOut": 100, "beepStart": true, "eventUrl": ["http://example.com"], "eventMethod": "PUT"}'

record_url_as_str = '{"action": "record", "eventUrl": ["http://example.com/events"]}'

record_add_split = '{"action": "record", "split": "conversation", "channels": 4}'

conversation_basic = '{"action": "conversation", "name": "my_conversation"}'

conversation_full = '{"action": "conversation", "name": "my_conversation", "musicOnHoldUrl": ["http://example.com/music.mp3"], "startOnEnter": true, "endOnExit": true, "record": true, "canSpeak": ["asdf", "qwer"], "canHear": ["asdf"]}'

conversation_mute_option = '{"action": "conversation", "name": "my_conversation", "mute": true}'

connect_phone = '{"action": "connect", "endpoint": [{"type": "phone", "number": "447000000000", "dtmfAnswer": "1p2p3p#**903#", "onAnswer": {"url": "https://example.com/answer", "ringbackTone": "http://example.com/ringbackTone.wav"}}]}'

connect_app = '{"action": "connect", "endpoint": [{"type": "app", "user": "test_user"}]}'

connect_websocket = '{"action": "connect", "endpoint": [{"type": "websocket", "uri": "ws://example.com/socket", "contentType": "audio/l16;rate=8000", "headers": {"language": "en-GB"}}]}'

connect_sip = '{"action": "connect", "endpoint": [{"type": "sip", "uri": "sip:rebekka@sip.mcrussell.com", "headers": {"location": "New York City", "occupation": "developer"}}]}'

connect_vbc = '{"action": "connect", "endpoint": [{"type": "vbc", "extension": "111"}]}'

connect_full = '{"action": "connect", "endpoint": [{"type": "phone", "number": "447000000000"}], "from": "447400000000", "randomFromNumber": false, "eventType": "synchronous", "timeout": 15, "limit": 1000, "machineDetection": "hangup", "eventUrl": ["http://example.com"], "eventMethod": "PUT", "ringbackTone": "http://example.com"}'

connect_advancedMachineDetection = '{"action": "connect", "endpoint": [{"type": "phone", "number": "447000000000"}], "from": "447400000000", "advancedMachineDetection": {"behavior": "continue", "mode": "detect"}, "eventUrl": ["http://example.com"]}'

talk_basic = '{"action": "talk", "text": "hello"}'

talk_full = '{"action": "talk", "text": "hello", "bargeIn": true, "loop": 3, "level": 0.5, "language": "en-GB", "style": 1, "premium": true}'

stream_basic = '{"action": "stream", "streamUrl": ["https://example.com/stream/music.mp3"]}'

stream_full = '{"action": "stream", "streamUrl": ["https://example.com/stream/music.mp3"], "level": 0.1, "bargeIn": true, "loop": 10}'

input_basic_dtmf = '{"action": "input", "type": ["dtmf"]}'

input_basic_dtmf_speech = '{"action": "input", "type": ["dtmf", "speech"]}'

input_dtmf_and_speech_full = '{"action": "input", "type": ["dtmf", "speech"], "dtmf": {"timeOut": 5, "maxDigits": 12, "submitOnHash": true}, "speech": {"uuid": "my-uuid", "endOnSilence": 2.5, "language": "en-GB", "context": ["sales", "billing"], "startTimeout": 20, "maxDuration": 30, "saveAudio": true}, "eventUrl": ["http://example.com/speech"], "eventMethod": "PUT"}'

notify_basic = '{"action": "notify", "payload": {"message": "hello"}, "eventUrl": ["http://example.com"]}'

notify_full = (
    '{"action": "notify", "payload": {"message": "hello"}, "eventUrl": ["http://example.com"], "eventMethod": "POST"}'
)

pay_basic = '{"action": "pay", "amount": 10.0}'

pay_voice_full = '{"action": "pay", "amount": 99.99, "currency": "gbp", "eventUrl": ["https://example.com/payment"], "voice": {"language": "en-GB", "style": 1}}'

pay_text = '{"action": "pay", "amount": 12.35, "currency": "gbp", "eventUrl": ["https://example.com/payment"], "prompts": {"type": "CardNumber", "text": "Enter your card number.", "errors": {"InvalidCardType": {"text": "The card you are trying to use is not valid for this purchase."}}}}'

pay_text_multiple_prompts = '{"action": "pay", "amount": 12.0, "prompts": [{"type": "CardNumber", "text": "Enter your card number.", "errors": {"InvalidCardType": {"text": "The card you are trying to use is not valid for this purchase."}}}, {"type": "ExpirationDate", "text": "Enter your card expiration date.", "errors": {"InvalidExpirationDate": {"text": "You have entered an invalid expiration date."}, "Timeout": {"text": "Please enter your card\'s expiration date."}}}, {"type": "SecurityCode", "text": "Enter your 3-digit security code.", "errors": {"InvalidSecurityCode": {"text": "You have entered an invalid security code."}, "Timeout": {"text": "Please enter your card\'s security code."}}}]}'

two_notify_ncco = '[{"action": "notify", "payload": {"message": "hello"}, "eventUrl": ["http://example.com"]}, {"action": "notify", "payload": {"message": "world"}, "eventUrl": ["http://example.com"], "eventMethod": "PUT"}]'



from .errors import ProactiveConnectError

import requests
import logging
from typing import List

logger = logging.getLogger("vonage")


class ProactiveConnect:
    def __init__(self, client):
        self._client = client
        self._auth_type = 'jwt'

    def list_all_lists(self, page: int = None, page_size: int = None):
        params = self._check_pagination_params(page, page_size)
        return self._client.get(
            self._client.proactive_connect_host(),
            '/v0.1/bulk/lists',
            params,
            auth_type=self._auth_type,
        )

    def create_list(self, params: dict):
        self._validate_list_params(params)
        return self._client.post(
            self._client.proactive_connect_host(),
            '/v0.1/bulk/lists',
            params,
            auth_type=self._auth_type,
        )

    def get_list(self, list_id: str):
        return self._client.get(
            self._client.proactive_connect_host(),
            f'/v0.1/bulk/lists/{list_id}',
            auth_type=self._auth_type,
        )

    def update_list(self, list_id: str, params: dict):
        self._validate_list_params(params)
        return self._client.put(
            self._client.proactive_connect_host(),
            f'/v0.1/bulk/lists/{list_id}',
            params,
            auth_type=self._auth_type,
        )

    def delete_list(self, list_id: str):
        return self._client.delete(
            self._client.proactive_connect_host(),
            f'/v0.1/bulk/lists/{list_id}',
            auth_type=self._auth_type,
        )

    def clear_list(self, list_id: str):
        return self._client.post(
            self._client.proactive_connect_host(),
            f'/v0.1/bulk/lists/{list_id}/clear',
            params=None,
            auth_type=self._auth_type,
        )

    def sync_list_from_datasource(self, list_id: str):
        return self._client.post(
            self._client.proactive_connect_host(),
            f'/v0.1/bulk/lists/{list_id}/fetch',
            params=None,
            auth_type=self._auth_type,
        )

    def list_all_items(self, list_id: str, page: int = None, page_size: int = None):
        params = self._check_pagination_params(page, page_size)
        return self._client.get(
            self._client.proactive_connect_host(),
            f'/v0.1/bulk/lists/{list_id}/items',
            params,
            auth_type=self._auth_type,
        )

    def create_item(self, list_id: str, data: dict):
        params = {'data': data}
        return self._client.post(
            self._client.proactive_connect_host(),
            f'/v0.1/bulk/lists/{list_id}/items',
            params,
            auth_type=self._auth_type,
        )

    def get_item(self, list_id: str, item_id: str):
        return self._client.get(
            self._client.proactive_connect_host(),
            f'/v0.1/bulk/lists/{list_id}/items/{item_id}',
            auth_type=self._auth_type,
        )

    def update_item(self, list_id: str, item_id: str, data: dict):
        params = {'data': data}
        return self._client.put(
            self._client.proactive_connect_host(),
            f'/v0.1/bulk/lists/{list_id}/items/{item_id}',
            params,
            auth_type=self._auth_type,
        )

    def delete_item(self, list_id: str, item_id: str):
        return self._client.delete(
            self._client.proactive_connect_host(),
            f'/v0.1/bulk/lists/{list_id}/items/{item_id}',
            auth_type=self._auth_type,
        )

    def download_list_items(self, list_id: str, file_path: str) -> List[dict]:
        uri = f'https://{self._client.proactive_connect_host()}/v0.1/bulk/lists/{list_id}/items/download'
        logger.debug(
            f'GET request with Proactive Connect to {repr(uri)}, downloading items from list {list_id} to file {file_path}'
        )
        headers = {**self._client.headers, 'Authorization': self._client._create_jwt_auth_string()}
        response = requests.get(
            uri,
            headers=headers,
        )
        if 200 <= response.status_code < 300:
            with open(file_path, 'wb') as file:
                file.write(response.content)
        else:
            return self._client.parse(self._client.proactive_connect_host(), response)

    def upload_list_items(self, list_id: str, file_path: str):
        uri = f'https://{self._client.proactive_connect_host()}/v0.1/bulk/lists/{list_id}/items/import'
        with open(file_path, 'rb') as csv_file:
            logger.debug(
                f'POST request with Proactive Connect uploading {file_path} to {repr(uri)}'
            )
            headers = {
                **self._client.headers,
                'Authorization': self._client._create_jwt_auth_string(),
            }
            response = requests.post(
                uri,
                headers=headers,
                files={'file': ('list_items.csv', csv_file, 'text/csv')},
            )
        return self._client.parse(self._client.proactive_connect_host(), response)

    def list_events(self, page: int = None, page_size: int = None):
        params = self._check_pagination_params(page, page_size)
        return self._client.get(
            self._client.proactive_connect_host(),
            '/v0.1/bulk/events',
            params,
            auth_type=self._auth_type,
        )

    def _check_pagination_params(self, page: int = None, page_size: int = None) -> dict:
        params = {}
        if page is not None:
            if type(page) == int and page > 0:
                params['page'] = page
            elif page <= 0:
                raise ProactiveConnectError('"page" must be an int > 0.')
        if page_size is not None:
            if type(page_size) == int and page_size > 0:
                params['page_size'] = page_size
            elif page_size and page_size <= 0:
                raise ProactiveConnectError('"page_size" must be an int > 0.')
        return params

    def _validate_list_params(self, params: dict):
        if 'name' not in params:
            raise ProactiveConnectError('You must supply a name for the new list.')
        if (
            'datasource' in params
            and 'type' in params['datasource']
            and params['datasource']['type'] == 'salesforce'
        ):
            self._check_salesforce_params_correct(params['datasource'])

    def _check_salesforce_params_correct(self, datasource):
        if 'integration_id' not in datasource or 'soql' not in datasource:
            raise ProactiveConnectError(
                'You must supply a value for "integration_id" and "soql" when creating a list with Salesforce.'
            )
        if type(datasource['integration_id']) is not str or type(datasource['soql']) is not str:
            raise ProactiveConnectError(
                'You must supply values for "integration_id" and "soql" as strings.'
            )



import io
import os

from setuptools import setup, find_packages


with io.open(os.path.join(os.path.dirname(__file__), "README.md"), encoding="utf-8") as f:
    long_description = f.read()

setup(
    name="vonage",
    version="3.8.0",
    description="Vonage Server SDK for Python",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Vonage/vonage-python-sdk",
    author="Vonage",
    author_email="devrel@vonage.com",
    license="Apache",
    packages=find_packages(where="src"),
    package_dir={"": "src"},
    platforms=["any"],
    install_requires=[
        "vonage-jwt>=1.0.0",
        "requests>=2.4.2",
        "pytz>=2018.5",
        "Deprecated",
        "pydantic>=1.10,==1.*",
    ],
    python_requires=">=3.7",
    tests_require=["cryptography>=2.3.1"],
    classifiers=[
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Programming Language :: Python :: 3.10",
        "Programming Language :: Python :: 3.11",
    ],
)



from pydantic import BaseModel, HttpUrl, AnyUrl, Field, constr
from typing import Optional, Dict
from typing_extensions import Literal


class ConnectEndpoints:
    class Endpoint(BaseModel):
        type: str = None

    class PhoneEndpoint(Endpoint):
        type = Field('phone', const=True)
        number: constr(regex=r'^[1-9]\d{6,14}$')
        dtmfAnswer: Optional[constr(regex='^[0-9*#p]+$')]
        onAnswer: Optional[Dict[str, HttpUrl]]

    class AppEndpoint(Endpoint):
        type = Field('app', const=True)
        user: str

    class WebsocketEndpoint(Endpoint):
        type = Field('websocket', const=True)
        uri: AnyUrl
        contentType: Literal['audio/l16;rate=16000', 'audio/l16;rate=8000']
        headers: Optional[dict]

    class SipEndpoint(Endpoint):
        type = Field('sip', const=True)
        uri: str
        headers: Optional[dict]

    class VbcEndpoint(Endpoint):
        type = Field('vbc', const=True)
        extension: str

    @classmethod
    def create_endpoint_model_from_dict(cls, d) -> Endpoint:
        if d['type'] == 'phone':
            return cls.PhoneEndpoint.parse_obj(d)
        elif d['type'] == 'app':
            return cls.AppEndpoint.parse_obj(d)
        elif d['type'] == 'websocket':
            return cls.WebsocketEndpoint.parse_obj(d)
        elif d['type'] == 'sip':
            return cls.WebsocketEndpoint.parse_obj(d)
        elif d['type'] == 'vbc':
            return cls.WebsocketEndpoint.parse_obj(d)
        else:
            raise ValueError(
                'Invalid "type" specified for endpoint object. Cannot create a ConnectEndpoints.Endpoint model.'
            )
